"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var typedoc_1 = require("typedoc");
var converter_1 = require("typedoc/dist/lib/converter");
var events_1 = require("typedoc/dist/lib/output/events");
var enumerations_1 = require("./enumerations");
var image_generator_1 = require("./image_generator");
var caching_plantuml_generator_1 = require("./plantuml/caching_plantuml_generator");
var plantuml_utils_1 = require("./plantuml/plantuml_utils");
var plugin_options_1 = require("./plugin_options");
var page_processor_1 = require("./typedoc/page_processor");
var page_section_1 = require("./typedoc/page_section");
var page_section_finder_1 = require("./typedoc/page_section_finder");
var typedoc_utils_1 = require("./typedoc/typedoc_utils");
/**
 * The UML class diagram generator plugin.
 *
 * # What does it do?
 *
 * This plugin generates an UML class diagram for classes and interfaces.
 *
 * # How does it do it?
 *
 * 1. The plugin scans through the classes and interfaces and generates PlantUML code for them.
 * 2. The plugin uses the node-plantuml module to generate image files from the PlantUML code.
 * 3. The plugin adds a section with the image to the page of each class and interface.
 */
var Plugin = /** @class */ (function () {
    function Plugin() {
        /** The options of this plugin. */
        this.options = new plugin_options_1.PluginOptions();
        /** Used when the class diagrams are created locally. */
        this.localImageGenerator = new image_generator_1.ImageGenerator();
    }
    /**
     * Initializes the plugin.
     * @param typedoc The TypeDoc application.
     */
    Plugin.prototype.initialize = function (typedoc) {
        this.addOptionsToApplication(typedoc);
        this.subscribeToApplicationEvents(typedoc);
    };
    /**
     * Adds the plugin's options to the application's options.
     * @param typedoc The TypeDoc application.
     */
    Plugin.prototype.addOptionsToApplication = function (typedoc) {
        this.options.addToApplication(typedoc);
    };
    /**
     * Subscribes to events of the application so that the plugin can do its work
     * in the particular doc generation phases.
     * @param typedoc The TypeDoc application.
     */
    Plugin.prototype.subscribeToApplicationEvents = function (typedoc) {
        var _this = this;
        typedoc.converter.on(converter_1.Converter.EVENT_RESOLVE_BEGIN, function (c) { return _this.onConverterResolveBegin(c); });
        typedoc.renderer.on(events_1.RendererEvent.BEGIN, function (e) { return _this.onRendererBegin(e); });
        typedoc.renderer.on(events_1.PageEvent.END, function (e) { return _this.onRendererEndPage(e); });
        typedoc.renderer.on(events_1.RendererEvent.END, function (e) { return _this.onRendererEnd(e); });
    };
    /**
     * Triggered when the TypeDoc converter begins resolving a project.
     * Reads plugin parameter values.
     * @param context Describes the current state the converter is in.
     */
    Plugin.prototype.onConverterResolveBegin = function (context) {
        this.options.readValuesFromApplication(context.converter.owner.application);
        this.plantUmlGenerator = new caching_plantuml_generator_1.CachingPlantUmlGenerator(this.options);
    };
    /**
     * Triggered before the renderer starts rendering a project.
     * Setup image generator output directory and progress bar.
     * @param event The event emitted by the renderer class.
     */
    Plugin.prototype.onRendererBegin = function (event) {
        this.localImageGenerator.setOutputDirectory(path.join(event.outputDirectory, "assets/images/"));
    };
    /**
     * Triggered after a document has been rendered, just before it is written to disc.
     * Generates a UML class diagram and adds it to the page of the reflection.
     * @param event The event emitted by the renderer class.
     */
    Plugin.prototype.onRendererEndPage = function (event) {
        if (this.shouldProcessPage(event.contents, event.model)) {
            this.processPage(event);
        }
    };
    /**
     * Checks if the plugin should process a given page.
     * @param pageContent The content of the page.
     * @param pageModel The model for the page.
     * @returns True, if the plugin should process the page, otherwise false.
     */
    Plugin.prototype.shouldProcessPage = function (pageContent, pageModel) {
        var isPluginActive = this.options.umlClassDiagramType === enumerations_1.ClassDiagramType.Simple ||
            this.options.umlClassDiagramType === enumerations_1.ClassDiagramType.Detailed;
        if (isPluginActive && pageContent) {
            var modelIsClassOrInterface = pageModel instanceof typedoc_1.DeclarationReflection &&
                (pageModel.kind === typedoc_1.ReflectionKind.Class || pageModel.kind === typedoc_1.ReflectionKind.Interface);
            if (modelIsClassOrInterface) {
                var reflection = pageModel;
                if (typedoc_utils_1.TypeDocUtils.reflectionIsPartOfClassHierarchy(reflection) &&
                    page_section_finder_1.PageSectionFinder.hasSection(pageContent, "Hierarchy" /* Hierarchy */)) {
                    return true;
                }
            }
        }
        return false;
    };
    /**
     * Processes the page by inserting a hierarchy diagram into it.
     * @param event The page event with the page data.
     */
    Plugin.prototype.processPage = function (event) {
        var imageUrl = this.createHierarchyDiagramForPage(event);
        this.insertHierarchyDiagramIntoPage(event, imageUrl);
    };
    /**
     * Creates a hierarchy diagram for the page.
     * @param event The page event with the page data.
     * @returns The URL to the hierarchy diagram.
     */
    Plugin.prototype.createHierarchyDiagramForPage = function (event) {
        var reflection = event.model;
        var plantUmlLines = this.plantUmlGenerator.createClassDiagramPlantUmlForReflection(reflection);
        var encodedPlantUml = plantuml_utils_1.PlantUmlUtils.encode(plantUmlLines.join("\n"));
        var imageUrl = "";
        if (this.options.outputImageLocation === plugin_options_1.ImageLocation.Local) {
            var absoluteImagePath = this.localImageGenerator.writeImageFile(encodedPlantUml, reflection.name, this.options.outputImageFormat);
            imageUrl = path.relative(path.dirname(event.filename), absoluteImagePath);
        }
        else {
            imageUrl = plantuml_utils_1.PlantUmlUtils.createPlantUmlServerUrl(encodedPlantUml, this.options.outputImageFormat);
        }
        return imageUrl;
    };
    /**
     * Inserts the hierarchy diagram into the page.
     * @param event The page event with the page data.
     * @param imageUrl The URL to the hierarchy diagram.
     */
    Plugin.prototype.insertHierarchyDiagramIntoPage = function (event, imageUrl) {
        var reflection = event.model;
        var hierarchyDiagramSection = page_section_1.PageSection.createForHierarchyDiagram(this.options.umlClassDiagramSectionTitle, imageUrl, reflection.name);
        var page = new page_processor_1.PageProcessor(event.contents);
        if (this.options.umlClassDiagramPosition === plugin_options_1.ClassDiagramPosition.Above) {
            page.insertAboveSection("Hierarchy" /* Hierarchy */, hierarchyDiagramSection);
        }
        else {
            page.insertBelowSection("Hierarchy" /* Hierarchy */, hierarchyDiagramSection);
        }
        event.contents = page.content;
    };
    /**
     * Triggered after the renderer has written all documents.
     * Appends style data to the main CSS file.
     * @param event The event emitted by the renderer class.
     */
    Plugin.prototype.onRendererEnd = function (event) {
        var filename = path.join(event.outputDirectory, "assets/css/main.css");
        var data = fs.readFileSync(filename, "utf8") +
            "\n.uml-class { max-width:100%; display:block; margin:0 auto; text-align:center }\n";
        fs.writeFileSync(filename, data, "utf8");
    };
    return Plugin;
}());
exports.Plugin = Plugin;
